package net.macpod.robocomm;

import java.io.DataInputStream;
import java.net.InetAddress;
import java.net.Socket;
import android.media.AudioManager;
import android.media.ToneGenerator;
import android.os.Handler;
import android.os.Message;

/**
 * Allows electronics to be controlled via dtmf tones over the headphone jack of
 * an android device. These are supplied by through a server this class connects
 * to.
 * 
 * @author Jeffrey Nelson <nelsonjm@macpod.net> http://macpod.net
 * @date 1/20/09
 * @version 1.0
 * Licensed under the GPL v2, see the file LICENSE in this tarball.
 */
public class RoboClient {
	private RoboClientThreader threader = null;

	/**
	 * Connects to a given host on the specified port number
	 * 
	 * @param host
	 *            Name or ip of host to connect to.
	 * @param port
	 *            Port number to connect to on host.
	 * @param new_handler
	 *            Handler used to change the state of an applications interface.
	 * @return true if the connection was setup fine, false otherwise.
	 */
	public boolean connect(String host, int port, Handler new_handler) {
		if (threader != null && threader.connected()) {
			return false;
		}
		try {
			threader = new RoboClientThreader();
			return threader.connect(host, port, new_handler);
		} catch (Exception e) {
			return false;
		}
	}

	/**
	 * Disconnects from a control server if connected, or does nothing
	 * otherwise.
	 */
	public void disconnect() {
		try {
			if (threader == null)
				return;
			threader.disconnect();
			threader = null;
		} catch (Exception e) {
		}
	}

	/**
	 * May be used to check the state of a connection with a server.
	 * 
	 * @return true if connected, false otherwise.
	 */
	public boolean connected() {
		if (threader == null)
			return false;
		return threader.connected();
	}

	/**
	 * This private class is wrapped within another class so that the run()
	 * method cannot be directaly accessed. It controls the real gruntwork of
	 * connecting to a control server and generating dtmf tones.
	 * 
	 * @author Jeffrey Nelson <nelsonjm@macpod.net>
	 * 
	 */
	private class RoboClientThreader extends Thread {
		/**
		 * Specifies the minimum duration the DTMF tone should be generated for.
		 */
		public static final int DTMF_INTERVAL = 40;

		private Socket connection = null;
		private Handler interface_handler;

		/**
		 * Update client's interface, process server data, closes shop, and
		 * updates the clients interface again.
		 */
		public void run() {
			ToneGenerator dtmfs = null;

			// Update the client's interface to show we are running.
			try {
				Message m = new Message();
				m.what = RoboComm.CONNECTED_INTERFACE;
				interface_handler.sendMessage(m);
			} catch (Exception e) {
			}

			try {
				// Create the tone generator.
				dtmfs = new ToneGenerator(AudioManager.STREAM_MUSIC,
						ToneGenerator.MAX_VOLUME);
				DataInputStream in = new DataInputStream(connection
						.getInputStream());
				/**
				 * Continue reading forever.. this loop exits when we get an
				 * exception related to the socket input stream closing.
				 */
				while (true) {
					try {
						int val = in.readByte();
						switch (val) {
						case '0':
							dtmfs.startTone(ToneGenerator.TONE_DTMF_0);
							break;
						case '1':
							dtmfs.startTone(ToneGenerator.TONE_DTMF_1);
							break;
						case '2':
							dtmfs.startTone(ToneGenerator.TONE_DTMF_2);
							break;
						case '3':
							dtmfs.startTone(ToneGenerator.TONE_DTMF_3);
							break;
						case '4':
							dtmfs.startTone(ToneGenerator.TONE_DTMF_4);
							break;
						case '5':
							dtmfs.startTone(ToneGenerator.TONE_DTMF_5);
							break;
						case '6':
							dtmfs.startTone(ToneGenerator.TONE_DTMF_6);
							break;
						case '7':
							dtmfs.startTone(ToneGenerator.TONE_DTMF_7);
							break;
						case '8':
							dtmfs.startTone(ToneGenerator.TONE_DTMF_8);
							break;
						case '9':
							dtmfs.startTone(ToneGenerator.TONE_DTMF_9);
							break;
						default:
							break;
						}
						/**
						 * Wait the specified number of milliseconds before
						 * trying to stop the tone.
						 */
						sleep(DTMF_INTERVAL);
						dtmfs.stopTone();
					} catch (InterruptedException interrupt) {
						// Sleep can throw this exception, ignore it.
					}
				}
			} catch (Exception e) {
				/*
				 * Try to everything up, even if it has already been done
				 * before.
				 */
				try {
					dtmfs.release();
				} catch (Exception ee) {
				}
				try {
					connection.shutdownInput();
				} catch (Exception ee) {
				}
				try {
					connection.shutdownOutput();
				} catch (Exception ee) {
				}
				try {
					connection.close();
				} catch (Exception ee) {
				}
			} finally {
				connection = null;
				/*
				 * Update the interface to refect we are no longer connected.
				 */
				try {
					Message m = new Message();
					m.what = RoboComm.DISCONNECTED_INTERFACE;
					interface_handler.sendMessage(m);
				} catch (Exception e) {
				}
			}
		}

		public boolean connected() {
			return (connection != null);
		}

		public boolean connect(String host, int port, Handler new_handler) {
			boolean err = false;
			if (connected()) {
				return false;
			}

			try {
				InetAddress addr = InetAddress.getByName(host);
				connection = new Socket(addr, port);
				interface_handler = new_handler;
				start();
			} catch (Exception e) {
				connection = null;
				err = true;
			}

			return !err;
		}

		public void disconnect() {
			if (connection == null)
				return;

			try {
				connection.shutdownInput();
			} catch (Exception e) {
			}
			try {
				connection.shutdownOutput();
			} catch (Exception e) {
			}
			try {
				connection.close();
			} catch (Exception e) {
			}
			try {
				join();
			} catch (Exception e) {
			}
		}
	}
}
