/*
Author: Jeffrey Nelson
 	nelsonjm@macpod.net
 	http://macpod.net
 Date: 1/20/09
 Description: Using a DTMF MT8870 IC, decodes DTMF tones then controls two
 	servos directly and two motors using a SN754410 quad half h-bridge.
 
 	
 Licensed under the GPL v2, see the file LICENSE in this tarball.
 */

#include <Servo.h>
//#include <SoftwareSerial.h>

// DTMF VALUES
#define DTMF_1 1
#define DTMF_2 2
#define DTMF_3 3
#define DTMF_4 4
#define DTMF_5 5
#define DTMF_6 6
#define DTMF_7 7
#define DTMF_8 8
#define DTMF_9 9
#define DTMF_0 10
#define DTMF_STAR 11
#define DTMF_POUND 12
#define DTMF_A 13
#define DTMF_B 14
#define DTMF_C 15
#define DTMF_D 16

#define LED_PIN 13

// Interrupt 0 is on pin 2
#define DTMF_STD_EXTN_INTR 0

// DTMF circuit input pins for each bit of the nibble (4 bits)
#define DTMF_Q4_PIN 4
#define DTMF_Q3_PIN 7
#define DTMF_Q2_PIN 8
#define DTMF_Q1_PIN 12

// Servo data pins for left/right arms.
#define L_ARM_PIN 9
#define R_ARM_PIN 10

// Left motor control
#define L_FORWARD 3
#define L_REVERSE 5

// Right motor control
#define R_FORWARD 6
#define R_REVERSE 11

// Servo objects used to control the left/right arms (or fork and knife)
Servo r_arm;
Servo l_arm;

volatile bool dtmf_flag = false; // This flag is set when new tone is detected
byte dtmf_val = 0;

int speed=150; // Set pwm value to control robot speed.
char state;

// Writes to pins connected to h-bridge to make robot stop
void stop()
{
  digitalWrite(L_FORWARD, LOW);
  digitalWrite(R_FORWARD, LOW);
  digitalWrite(L_REVERSE, LOW);
  digitalWrite(R_REVERSE, LOW);
}

// Writes to pins connected to h-bridge to make robot move forward
void forward()
{
  digitalWrite(L_REVERSE, LOW); 
  digitalWrite(R_REVERSE,LOW);   
  analogWrite(L_FORWARD, speed);
  analogWrite(R_FORWARD, speed);     
}

// Writes to pins connected to h-bridge to make robot move backwards
void reverse()
{
  digitalWrite(L_FORWARD , LOW); 
  digitalWrite(R_FORWARD, LOW);     
  analogWrite(R_REVERSE, speed);
  analogWrite(L_REVERSE, speed);                      
}

// Writes to pins connected to h-bridge to make robot turn left
void left()
{
  digitalWrite(L_FORWARD, LOW);
  digitalWrite(R_REVERSE, LOW);  
  analogWrite(L_REVERSE, speed);
  analogWrite(R_FORWARD, speed);
}

// Writes to pins connected to h-bridge to make robot turn right
void right()
{
  digitalWrite(L_REVERSE, LOW);
  digitalWrite(R_FORWARD, LOW);
  analogWrite(L_FORWARD, speed);
  analogWrite(R_REVERSE, speed);
}





// This function is called when the DTMF decoder chip's pin
// goes "high" indicating a new tone has been detected.
void dtmf_interrupt()
{
  dtmf_flag = true;
}

// Converts the nibble (4 bits) given to us from the DMTF chip and sticks them in a byte.
byte dtmf_nibble()
{
  return B00000000 | 
    byte(digitalRead(DTMF_Q1_PIN)==HIGH) |
    byte(digitalRead(DTMF_Q2_PIN)==HIGH) << 1 |
    byte(digitalRead(DTMF_Q3_PIN)==HIGH) << 2 |
    byte(digitalRead(DTMF_Q4_PIN)==HIGH) << 3;

}

void setup() {
  //Serial.begin(9600);	// opens serial port, sets data rate to 9600 bps
  //Serial.println("DTMF Robot Controller");

  pinMode(LED_PIN, OUTPUT);

  // Set pins connected to DTMF chip to be input pins.
  pinMode(DTMF_Q1_PIN, INPUT);
  pinMode(DTMF_Q2_PIN, INPUT);
  pinMode(DTMF_Q3_PIN, INPUT);
  pinMode(DTMF_Q4_PIN, INPUT);

  // Set pins connected to the h-bridge to be output pins.
  pinMode(L_FORWARD, OUTPUT);
  pinMode(L_REVERSE, OUTPUT);
  pinMode(R_FORWARD, OUTPUT);
  pinMode(R_REVERSE, OUTPUT); 

  // Set all h-bridge out pins to low.
  stop(); 

  // Connect the servo controller objects to the pins that the data line is on for the 
  // left and right arms.
  r_arm.attach(R_ARM_PIN);
  l_arm.attach(L_ARM_PIN);

  // Attaches an interrupt to the DTMF pin that lets us know when a new tone is avaliable.
  // This function is called when the pin's state changes from low to high (aka RISING)
  attachInterrupt( DTMF_STD_EXTN_INTR, dtmf_interrupt, RISING );
}

// Continuously look for and 
void loop() {

  if (dtmf_flag) {
    dtmf_val = dtmf_nibble();
    dtmf_flag = false;
    //Serial.print("New:");
    //Serial.println(dtmf_val, BIN);


    switch (dtmf_val) {
    case DTMF_1: // Raise left arm
      l_arm.write(165);
      break;
    case DTMF_7: // Lower left arm
      l_arm.write(10);
      break;

    case DTMF_3:  // Raise right arm
      r_arm.write(10);
      break;
    case DTMF_9:  // Lower right arm
      r_arm.write(165);
      break;

    case DTMF_2: // Forward
      forward();
      break;      
    case DTMF_5: // Stop
      stop();
      break;          
    case DTMF_8: // Reverse
      reverse();
      break; 
    case DTMF_4: // Left
      left();
      break; 
    case DTMF_6: // Right
      right();
      break;         

    default:
      break;
    }
  }

}
